"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const child_process_1 = require("child_process");
const xml2json_1 = require("xml2json");
const logger_1 = require("./logger");
class AppHostConfig {
    constructor() {
        this.iisDirectory = path_1.normalize('C:/Windows/System32/inetsrv/');
        this.appHostConfigDirectory = path_1.join(this.iisDirectory, 'Config/');
        this.appHostConfigPath = path_1.join(this.appHostConfigDirectory, 'applicationHost.config');
        this._config = fs_1.readFileSync(this.appHostConfigPath);
        if (!this._config) {
            const error = 'There is no IIS installed.';
            logger_1.Logger.error(error, 'application-host-config');
            throw new Error(error);
        }
        this.$parseConfig();
    }
    async getSiteNames() {
        return (await this._execCommand(['list', 'site', '/text:name']))
            .filter((site) => Boolean(site));
    }
    getSite(siteName) {
        return this._getNodeBy(this.sites, (item) => item.name === siteName);
    }
    getSiteApplication(siteName) {
        const site = this.getSite(siteName);
        if (!site) {
            const error = `Site ${siteName} not found in IIS.`;
            logger_1.Logger.error(error, 'application-host-config');
            throw new ReferenceError(error);
        }
        const application = this._getNodeBy(site.application, (item) => item.path === '/');
        if (!application) {
            const error = `There is something weird with ${siteName} config.`;
            logger_1.Logger.error(error, 'application-host-config');
            throw new ReferenceError(error);
        }
        const virtualDirectory = this._getNodeBy(application.virtualDirectory, (item) => item.path === '/');
        if (!virtualDirectory) {
            const error = `There is something weird with ${siteName} config.`;
            logger_1.Logger.error(error, 'application-host-config');
            throw new ReferenceError(error);
        }
        return {
            path: application.path,
            pool: application.applicationPool,
            physicalPath: virtualDirectory.physicalPath
        };
    }
    async addSiteApplication(siteName, application) {
        if (this.existsSiteApplication(siteName, application.path)) {
            const error = `${application.path} application already exists at ${siteName} IIS site.`;
            logger_1.Logger.error(error, 'application-host-config');
            return;
        }
        await this._execCommand([
            'add',
            'app',
            `/site.name:${siteName}`,
            `/path:${application.path}`,
            `/physicalPath:${application.physicalPath}`
        ]);
        await this._execCommand([
            'set',
            'app',
            `${siteName}${application.path}`,
            `/applicationPool:${application.pool}`
        ]);
        logger_1.Logger.info(`${application.path} application has been added to ${siteName} IIS site.`, 'application-host-config');
    }
    async removeSiteApplication(siteName, applicationPath) {
        await this._execCommand(['delete', 'app', `${siteName}${applicationPath}`]);
        logger_1.Logger.info(`${applicationPath} application has been removed from ${siteName} IIS site.`, 'application-host-config');
    }
    existsSiteApplication(siteName, applicationPath) {
        const site = this.getSite(siteName);
        if (!site) {
            const error = `Site ${siteName} not found in IIS.`;
            logger_1.Logger.error(error, 'application-host-config');
            throw new ReferenceError(error);
        }
        let applications = site.application;
        if (!Array.isArray(applications)) {
            applications = [applications];
        }
        return Boolean(applications.filter((item) => item.path === applicationPath).length);
    }
    saveBackup() {
        let bakFileName = 'applicationHost.config.bak';
        try {
            fs_1.statSync(path_1.join(this.appHostConfigDirectory, bakFileName));
            let index = 0;
            while (true) {
                bakFileName = `applicationHost.config.${index++}.bak`;
                fs_1.statSync(path_1.join(this.appHostConfigDirectory, bakFileName));
            }
            // tslint:disable-next-line:no-empty
        }
        catch { }
        const bakPath = path_1.join(this.appHostConfigDirectory, bakFileName);
        fs_1.copyFileSync(this.appHostConfigPath, bakPath);
        logger_1.Logger.info(`Config backup has been saved to ${bakPath}.`, 'application-host-config');
    }
    $parseConfig() {
        this.appHostConfig = JSON.parse(xml2json_1.toJson(String(this._config), {
            reversible: true,
            trim: false
        }));
        if (!this.appHostConfig) {
            const error = `IIS application host config can't be parsed.`;
            logger_1.Logger.error(error, 'application-host-config');
            throw new ReferenceError(error);
        }
        this.sites = this.appHostConfig.configuration['system.applicationHost'].sites.site;
        if (!Array.isArray(this.sites)) {
            this.sites = [this.sites];
        }
    }
    _getNodeBy(node, iteratee) {
        if (Array.isArray(node)) {
            return node.filter(iteratee).pop();
        }
        if (iteratee(node)) {
            return node;
        }
        return undefined;
    }
    async _execCommand(args) {
        const commands = args; // .map((arg) => arg.replace(/\\/g, '\\\\').replace(/\//g, '\\\/'));
        return new Promise((promiseResolve, promiseReject) => {
            child_process_1.execFile(path_1.join(this.iisDirectory, 'appcmd.exe'), commands, (error, stdout, stderr) => {
                if (error) {
                    logger_1.Logger.error(error.message, 'application-host-config');
                }
                if (stderr) {
                    logger_1.Logger.error(stderr, 'application-host-config');
                }
                if (error || stderr) {
                    promiseReject();
                }
                promiseResolve(String(stdout).split('\r\n'));
            });
        });
    }
}
exports.AppHostConfig = AppHostConfig;
//# sourceMappingURL=app-host-config.js.map