"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = require("path");
const node_windows_1 = require("node-windows");
const prompts_1 = tslib_1.__importDefault(require("prompts"));
require("colors");
const app_host_config_1 = require("./app-host-config");
const logger_1 = require("./logger");
class DirToIisAppService {
    constructor(argv) {
        this.argv = argv;
        this._config = new app_host_config_1.AppHostConfig();
    }
    async install() {
        this._config.saveBackup();
        await this.readOptions(await this.promptOptions());
    }
    async uninstall() {
        const options = (await this.promptOptions()).filter((option) => option.name === 'name');
        await this.readOptions(options);
    }
    async promptOptions() {
        return [
            {
                type: 'text',
                name: 'name',
                message: 'Service name',
                validate: (name) => /^[a-zA-Z\-]+$/.test(name) || 'Name must be only letters or dashes'
            },
            {
                type: 'select',
                name: 'site',
                message: 'IIS website name',
                choices: (await this._config.getSiteNames()).map((site) => ({ title: site, value: site }))
            },
            {
                type: 'text',
                name: 'paths',
                message: 'Paths to web.config files to be watched recursively, or glob patterns'
            },
            {
                type: 'number',
                name: 'interval',
                message: 'Interval of file system polling, in milliseconds.',
                initial: 15000,
                min: 1,
                increment: 1000,
                validate: (interval) => (interval > 0) || `Should be greater than 0.`
            },
            {
                type: 'number',
                name: 'depth',
                message: 'Limits how many levels of subdirectories will be traversed? (Leave -1 to unlimited)',
                initial: -1,
                min: -1,
                validate: (depth) => (depth >= -1) || `Should be greater than or equal to -1.`
            },
            {
                type: 'toggle',
                name: 'startImmediately',
                message: 'Should the service get started immediately?',
                initial: true,
                active: 'yes',
                inactive: 'no'
            }
        ];
    }
    async readOptions(options) {
        prompts_1.default.override(this.argv);
        this.run(await prompts_1.default(options));
    }
    run({ name, startImmediately, ...options }) {
        const serviceOptions = {
            name,
            description: `Directories watcher by pattern to add it to IIS website ${options.site} as applications.`,
            script: path_1.resolve(__dirname, './run-watcher.js'),
            scriptOptions: Object
                .entries(options)
                .map(([key, value]) => `--${key}=${value}`)
                .join(' ')
        };
        const service = new node_windows_1.Service(serviceOptions);
        this._bindEvents(service, startImmediately);
        service[this.argv.uninstall ? 'uninstall' : 'install']();
        logger_1.Logger.info(`Service ${this.argv.uninstall ? 'uninstall' : 'install'}ed: ${JSON.stringify(serviceOptions)}.`, 'service');
    }
    _bindEvents(service, startImmediately) {
        service.on('alreadyInstalled', () => console.log('Service already installed.'.cyan));
        service.on('invalidInstallation', () => console.log('Invalid service installation (installation is detected but required files are missing).'.red));
        service.on('uninstall', () => console.log('Service uninstalled.'.blue));
        service.on('start', () => console.log('Service started.'.green));
        service.on('stop', () => console.log('Service stopped.'.yellow));
        service.on('error', () => console.log('An error occurred.'.red));
        service.on('install', () => {
            console.log('Service installed.'.green);
            if (startImmediately) {
                service.start();
            }
        });
    }
}
exports.DirToIisAppService = DirToIisAppService;
//# sourceMappingURL=dir-to-iis-app-service.js.map